function [svm,obj,LOG]=algroupsvmclass(X,Y,options)
% USAGE [svm]=lgroupsvmclass(X,Y,options)
%
% linear svm for sparse group norm
%   min  C/N\sum_i max(1-yi(x_i*w+w0))^2  + Omega(w)
%   w,w0
% INPUT
%
% Training set
%      X  		: input data
%      Y  		: output data
% parameters
%       options.norm    :
%           - 1 : standard lasso
%           - 2 : group lasso \sum_g ||w_g||_2
%           - q : group l_q lasso \sum_g ||w_g||_q  with 1<q<2
%           - Inf : group l_infinite norm 
%		options.C		: regularisation term
%       options.groups  : groups for the group norm
%
% OUTPUT
%
% svm.w      weight
% svm.w0		bias
% svm.pos    position of Support Vector
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval, svmclass

%	01/02/2011 R. Flamary 
%       -adding other regularization
%	01/12/2009 R. Flamary 
%       -function creation
%       -use of accelerated gradient
%   05/11/2010 R. Flamary
%       - Add the Active set algorithm for large scale 


if ~isfield(options,'C'); options.C=1; end
if ~isfield(options,'norm'); options.norm=1; end
if ~isfield(options,'groups'); options.groups=1:size(X,2); end
if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'thr_prox'); options.thr_prox=1e-5; end

if length(unique(Y))>2
    options.multiclass=1;
end


if length(options.groups)~=size(X,2)
    error('wrong size for group vector')
end

if options.multiclass==0

  


loop=1;
nbloop=1;

% MAIN LOOP

%tic;

beta=ones(size(X,2),1);
x=beta;

  while loop
        %descente de gradient
        
        Xf=X*diag(beta);

        time(nbloop)=cputime;
        
        
        xold=x;

        
        
        [svm,obj,LOG]=glgroupsvmclass(Xf,Y,options);

        beta=get_group_norm(svm,options);
        
        x=svm.w;
        
        

        if nbloop>options.nbitermax2
            loop=0;
          if options.verbose  disp('Max number of iteration reached'); end
        end



        % J variation stopping criterion

            
        % x variation stopping criterion
        if norm(x)==0
            loop=0;
            if options.verbose disp('zero result'); end
        
        elseif max(max(max(abs(x-xold))))/sum(sum(sum(abs(x))))<options.stopvarx2
                  loop=0;
               if options.verbose disp('Delta x convergence'); end
       end


        nbloop=nbloop+1;
        
        options.W0=[svm.w;svm.w0];


  end

   

  svm.w=diag(beta)*svm.w;


else

    vals=sort(unique(Y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;

            for i=1:length(vals)


                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(Y==vals(i))-(Y~=vals(i));
                [models{i},obj(i),LOG{i}]=lgroupsvmclass(X,Yt,options2);

            end

            svm.multiclass=options.multiclass;
            svm.nbclass=length(vals);
            svm.models=models;
            svm.kerneloption=1;
            svm.kernel='linear';
            svm.vals=vals;




    end

end

end



function beta=get_group_norm(svm,options)

groups=options.groups;

vals=unique(groups);

beta=zeros(size(groups));

for i=1:length(vals)
    
   beta(groups==vals(i))=norm(svm.w(groups==vals(i)),options.norm) ;
end

end