function [W,b,obj]=mtlmklsvmProx(X,Y,lambda,options,verbose);


% Usage
%
% [W,b,obj]=mtlmklsvmProx(X,Y,lambda,options,verbose);

%  X : cells of input vector. 1 cell per task
%  Y : cells of outputs 1 cell per task
%  lambda : regularization parameters
%  options.norm_q :    should be equal to 2 for l1-ell2
%  options.seuildiffsigma : stopping criterion on variation of objective
%                               value
%  options.nbitermax : maximal number of iteration
%
%   note that we have implemented a version which X{k} are supposed to have the same dimension
%
%
%  outputs
%
%
%   W  : matrix of weights for linear decision functions
%   b  : bias
%   obj : objective value
%
%
%

% A. Rakotomamonjy 13/08/2010


nbtasks=length(Y);
dim=size(X{1},2);
W=zeros(dim,nbtasks);


V=W;
Wold=W+1;
az=1;
pas=0.001;
eta=0.5;
j=0;
T=3;

for k=1:nbtasks
    b(k)=0;
end;


while (j< T+1 | (max(obj(j-T:j))-min(obj(j-T:j)))/max(obj(j-T:j))> options.seuildiffsigma)& j < options.nbitermax;    % shrink

    normeW=(sum(abs(W).^options.norm_q,2).^(1/options.norm_q));
    normeW=sqrt(sum(W.^2,2)) ;    % shrink
    sumnorm = sum(lambda.*normeW);
    grad=zeros(dim,nbtasks);
    b=findBiasProx(X,Y,W);
    for k=1:nbtasks

        Loss(:,k)=1-Y{k}.*(X{k}*W(:,k));
        Loss(:,k)=Loss(:,k).*(Loss(:,k)>0);
        for i=1:size(Loss,1);
            if Loss(i,k) >0
                grad(:,k)=grad(:,k)-(X{k}(i,:)*Y{k}(i))';
            end;
        end;
    end;
    fobj=sum(sum(Loss));

    %
    % logistic regression
    %

    %     for k=1:nbtasks
    %         Loss(:,k)=log(1 + exp(-Y{k}.*(X{k}*W(:,k))));
    %         for i=1:size(Loss,1);
    %             grad(:,k)=grad(:,k)- (X{k}(i,:)*Y{k}(i)*exp(-Y{k}(i).*(X{k}(i,:)*W(:,k))))' /(1 + exp(-Y{k}(i).*(X{k}(i,:)*W(:,k))));
    %
    %         end;
    %     end;
    %     fobj=sum(sum(Loss));


    F=fobj+sumnorm;
    obj(j+1)=F;
    lin=trace((W-V)'*(grad));
    while F > fobj+sumnorm+lin + 1/2/pas*norm(W-V,'fro').^2;
        pas=pas*eta;
    end;
    Wold=W;
    W=V-pas*grad;   % gradient
    switch options.norm_q
        case 2
            normeW=sqrt(sum(W.^2,2));      % shrink
            bool=normeW>(lambda*pas);
            normeW(~bool)=eps;
            aux=(1-lambda*pas./normeW).*bool;
            W=W.*(aux*ones(1,nbtasks));           % shrink
        case 1
            W=(abs(W)-lambda*pas).*((abs(W)-lambda*pas)>0).*sign(W); % l1 solution

        otherwise
            if options.norm_q >2 || options.norm_q<1
                error('this q norm is not handled');
            else
                opt=[];
                qp=1/(1-1/options.norm_q);
                opt.threshold=options.seuildiffsigma;
                for k=1:dim
                    if norm(W(k,:),qp)<lambda*pas
                        W(k,:)=0;
                    else
                        opt.init=W(k,:)';
                        W(k,:)=LSquareLq(W(k,:)',options.norm_q,lambda*pas,opt)';
                    end;
                end;



            end;
    end;



    a1=2/( (j-1)+3);
    delta=W-Wold;
    V=W+(1-az)/az*a1*delta;
    az=a1;
    j=j+1;
    max(max(abs(W-Wold)));
end;

