function [svm,obj,LOG]=glgroupsvmclass(X,Y,options)
% USAGE [svm]=lgroupsvmclass(X,Y,options)
%
% linear svm for sparse group norm
%   min  C/N\sum_i max(1-yi(x_i*w+w0))^2  + Omega(w)
%   w,w0
% INPUT
%
% Training set
%      X  		: input data
%      Y  		: output data
% parameters
%       options.norm    :
%           - 1 : standard lasso
%           - 2 : group lasso \sum_g ||w_g||_2
%           - q : group l_q lasso \sum_g ||w_g||_q  with 1<q<2
%           - Inf : group l_infinite norm 
%		options.C		: regularisation term
%       options.groups  : groups for the group norm
%
% OUTPUT
%
% svm.w      weight
% svm.w0		bias
% svm.pos    position of Support Vector
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval, svmclass

%	01/02/2011 R. Flamary 
%       -adding other regularization
%	01/12/2009 R. Flamary 
%       -function creation
%       -use of accelerated gradient
%   05/11/2010 R. Flamary
%       - Add the Active set algorithm for large scale 


if ~isfield(options,'C'); options.C=1; end
if ~isfield(options,'norm'); options.norm=1; end
if ~isfield(options,'groups'); options.groups=1:size(X,2); end
if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'thr_prox'); options.thr_prox=1e-5; end


if length(unique(Y))>2
    options.multiclass=1;
end



% if options.norm==1
%     options.groups=1:size(X,2);
%     options.norm=2;
% end

options.popt=options.norm/(options.norm-1);

if options.norm==Inf
    options.popt=1;
end

if length(options.groups)~=size(X,2)
    error('wrong size for group vector')
end

if options.multiclass==0

    if nargin>3
        options.alphainit=alphainit;
    end

    options.C=options.C/length(Y);
    
    
    groups=options.groups;
    
    types=unique(options.groups);
    
    % put groups as 1,2 3,...,nbgroups
    for i=1:length(types)
       groups(groups==types(i))=i; 
    end
   
    options.groups=[options.groups(:);max(max(options.groups(:)))+1];
    
    data=options;
    
    data.X=[X, ones(size(X,1),1)];
    data.Y=Y;
    
    if ~isfield(options,'W0'); 
   % [w,w0] = lsvmclass(X,Y,options.C);
   % W0=[w;w0];
   % W0=randn(size(data.X,2),1);
        W0=zeros(size(data.X,2),1);
    else
        W0=options.W0;   
    end
    
    data.options=options;
    
    opts=options;
    opts.q_func=@q_func;
    opts.costfunc=@costfunc;
    
    if isequal(options.norm,'trace')
        opts.minq_func=@minq_func_trace;
        opts.q_func=@q_func_trace;
        opts.costfunc=@costfunc_trace;
    elseif options.norm==2 % norm 2
        opts.minq_func=@minq_func;
    elseif options.norm==1 % norm 1 
        opts.minq_func=@minq_func_1;
    elseif options.norm==Inf % norm infinie
        opts.minq_func=@minq_func_inf;    
    elseif options.norm>1 && options.norm<2 %n norm q
        opts.minq_func=@minq_func_q;   
    elseif options.norm>1 && options.norm<2 %n norm q
        opts.minq_func=@minq_func_q;   
    else
        error('this q norm is not handled');
    end
    
    [W,vars,obj,LOG]=nesterov_solver(W0,[],data,opts);

    LOG.constraint=get_constraints(W,data);
    
    svm.xsup=[];
    svm.w=W(1:end-1);
    svm.w0=W(end);
    svm.pos=[];
    svm.kerneloption=1;
    svm.kernel='linear';


else

    vals=sort(unique(Y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;

            for i=1:length(vals)


                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(Y==vals(i))-(Y~=vals(i));
                [models{i},obj(i),LOG{i}]=lgroupsvmclass(X,Yt,options2);

            end

            svm.multiclass=options.multiclass;
            svm.nbclass=length(vals);
            svm.models=models;
            svm.kerneloption=1;
            svm.kernel='linear';
            svm.vals=vals;




    end

end

end

function [costnew,varsnew]=q_func(W,Wt,vars,data)

    varsnew=vars;
    
    %V=W-gradf(W,vars,data)/vars.L;
    
    fw=data.C*1/2*sum(max(0,1-data.Y.*(data.X*Wt)).^2)+Omega1q_group(W(1:end-1),data.norm,data.groups(1:end-1));
    
    c2=sum( ( (W-Wt).*gradf(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);
    
    costnew=fw+c2;   

end

function [costnew,varsnew]=q_func_trace(W,Wt,vars,data)

    varsnew=vars;

    vals=unique(data.groups);

    
    for i=1:length(vals)-1
        V(:,i)=W(data.groups==vals(i));
    end
    
    [U,S,V] = svd(V,0);
    
    temp=sum((diag(S)));
    
    fw=data.C*1/2*sum(max(0,1-data.Y.*(data.X*Wt)).^2)+temp;
    
    c2=sum( ( (W-Wt).*gradf(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);
    
    costnew=fw+c2;   

end


function [costnew,varsnew]=costfunc_trace(W,vars,data)

    varsnew=vars;
    
      vals=unique(data.groups);

    
    for i=1:length(vals)-1
        V(:,i)=W(data.groups==vals(i));
    end
    
    [U,S,V] = svd(V,0);
    
    temp=sum((diag(S)));

    costnew=data.C*1/2*sum(max(0,1-data.Y.*(data.X*W)).^2)+temp;

end


function [costnew,varsnew]=costfunc(W,vars,data)

    varsnew=vars;

    costnew=data.C*1/2*sum(max(0,1-data.Y.*(data.X*W)).^2)+Omega1q_group(W(1:end-1),data.norm,data.groups(1:end-1));

end
function [W,obj,vars]=minq_func(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);

for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)))>lambda
        W(data.groups==vals(i))=(1-lambda/norm(Vt(data.groups==vals(i)))).*Vt(data.groups==vals(i));
    end
end

W(end)=Vt(end);

obj=0;

end

function [W,obj,vars]=minq_func_1(Vt,vars,data)

lambda=1/vars.L;

%W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

% l1 proximal
W=(1-lambda./abs(Vt)).*Vt.*(abs(Vt)>lambda);

W(end)=Vt(end);

obj=0;

end

function [W,obj,vars]=minq_func_inf(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);

% l-inf proximal see Chen 2009
for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)),data.popt)>lambda
        v=Vt(data.groups==vals(i));
        u=abs(v);
        u=sort(u,'descend');
        ur=cumsum(u);
        id=(1:length(v))';
        jvec=lambda-ur+id.*u;
        idtemp=find(jvec>0);
        val=(ur(idtemp(end))-lambda)/idtemp(end);
        W(data.groups==vals(i))=sign(v).*min(abs(v),val);
    end
end

W(end)=Vt(end);

obj=0;

end



function [W,obj,vars]=minq_func_q(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);
opt.threshold=data.thr_prox;
% l-inf proximal see Chen 2009
for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)),data.popt)>lambda

        W(data.groups==vals(i))=LSquareLq(Vt(data.groups==vals(i)),data.norm,lambda,opt);
    end
end

W(end)=Vt(end);

obj=0;

end



function [W,obj,vars]=minq_func_trace(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);

% l-inf proximal see Chen 2009
for i=1:length(vals)-1
    Vm(:,i)=Vt(data.groups==vals(i));
end

[S,V,D]=svd(Vm,'econ');

Wm=S*(sign(V).*max(0,V-lambda))*D';

for i=1:length(vals)-1
    W(data.groups==vals(i))=Wm(:,i);
end

W(end)=Vt(end);

obj=0;

end


function Df=gradf(W,vars,data)

    Xt=data.X.*repmat(data.Y,1,size(data.X,2));
   % Df=-data.C*sum(data.X.*repmat(data.Y.*max(0,1-data.Y.*(data.X*W)),1,size(data.X,2)))';
   Df=-data.C*Xt'*max(0,1-data.Y.*(data.X*W));
end


function constraints=get_constraints(W,data)

vals=unique(data.groups);

  for i=1:length(vals)-1  
      
      ri=data.C*(data.X(:,data.groups==vals(i)).*repmat(data.Y,1,sum(data.groups==vals(i))))'*max(0,1-data.Y.*(data.X*W));
     constraints(i)=norm(ri,data.popt);
  end

  
end





