function [svm,obj,LOG]=mtlgroupsvmclass(X,Y,options)
% USAGE [svm]=lgroupsvmclass(X,Y,options)
%
% linear svm for sparse group norm
%   min  C/N\sum_i max(1-yi(x_i*w+w0))^2  + Omega(w)
%   w,w0
% INPUT
%
% Training set
%      X  		: input data
%      Y  		: output data
% parameters
%       options.norm    :
%           - 1 : standard lasso
%           - 2 : group lasso \sum_g ||w_g||_2
%           - q : group l_q lasso \sum_g ||w_g||_q  with 1<q<2
%           - Inf : group l_infinite norm
%		options.C		: regularisation term
%       options.groups  : groups for the group norm
%
% OUTPUT
%
% svm.w      weight
% svm.w0		bias
% svm.pos    position of Support Vector
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval, svmclass

%	01/02/2011 R. Flamary
%       -adding other regularization
%	01/12/2009 R. Flamary
%       -function creation
%       -use of accelerated gradient
%   05/11/2010 R. Flamary
%       - Add the Active set algorithm for large scale


if ~isfield(options,'C'); options.C=1; end
if ~isfield(options,'norm'); options.norm=1; end
if ~isfield(options,'groups'); for i=1:length(Y);options.groups{i}=1:size(X{i},2);end; end
if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'thr_prox'); options.thr_prox=1e-5; end
if ~isfield(options,'reg_sim'); options.reg_sim=0; end

options.popt=options.norm/(options.norm-1);

if options.norm==Inf
    options.popt=1;
end

if options.norm==1
    gdeb=1;
    for i=1:length(Y)
        
       options.groups{i}=gdeb:gdeb+size(X{i},2)-1;
       gdeb=gdeb+size(X{i},2);
        
    end
    
end

Ctmp=options.C;
options.C=cell(0);
for i=1:length(Y)
    options.C{i}=Ctmp/length(Y{i});
end


% todo
for i=1:length(Y)
    options.C{i}=Ctmp/length(Y{i});
    gmax=max(options.groups{i});
end


options.gbias=gmax+1;
options.nbtasks=length(Y);options.nbtask=options.nbtasks;
data=options;

tasks=[];
groups=[];
for i=1:length(Y)
    tasks=[tasks i*ones(1,size(X{i},2)+1)];
    groups=[groups options.groups{i} gmax+1];
    data.X{i}=[X{i}, ones(size(X{i},1),1)];
    data.Y{i}=Y{i};
end
options.groups=groups;
options.tasks=tasks;
data.groups=groups;
data.tasks=tasks;

if ~isfield(options,'W0');
    % [w,w0] = lsvmclass(X,Y,options.C);
    % W0=[w;w0];
    % W0=randn(size(data.X,2),1);
    W0=zeros(length(groups),1);
else
    W0=options.W0;
end

data.options=options;

opts=options;
opts.q_func=@q_func;
opts.costfunc=@costfunc;

if options.norm==2 % norm 2
    opts.minq_func=@minq_func;
elseif options.norm==1 % norm 1
    opts.minq_func=@minq_func_1;
elseif options.norm==Inf % norm infinie
    opts.minq_func=@minq_func_inf;
elseif options.norm>1 && options.norm<2 %n norm q
    opts.minq_func=@minq_func_q;
else
    error('this q norm is not handled');
end

Linit=0;
for i=1:length(Y)
    Linit=max(Linit,norm(data.X{i}'*data.X{i})*options.C{i});
end
opts.L=Linit+options.reg_sim;

[W,vars,obj,LOG]=nesterov_solver(W0,[],data,opts);

LOG.constraint=get_constraints(W,data);


for i=1:length(Y)
    Wtemp=W(tasks==i);
    svm{i}.xsup=[];
    svm{i}.w=Wtemp(1:end-1);
    svm{i}.w0=Wtemp(end);
    svm{i}.pos=[];
    svm{i}.kerneloption=1;
    svm{i}.kernel='linear';
end


end

function [costnew,varsnew]=q_func(W,Wt,vars,data)

varsnew=vars;

%V=W-gradf(W,vars,data)/vars.L;

fw=0;
for i=1:data.nbtask
    fw=fw+data.C{i}*1/2*sum(max(0,1-data.Y{i}.*(data.X{i}*Wt(data.tasks==i))).^2);
end


c2=Omega1q_group(W(data.groups~=data.gbias),data.norm,data.groups(data.groups~=data.gbias))+ sum( ( (W-Wt).*gradf(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);

costnew=fw+c2;

if data.reg_sim
    costnew=costnew+data.reg_sim*cost_sim(Wt,data);
end

end

function [costnew,varsnew]=costfunc(W,vars,data)

varsnew=vars;

fw=0;
for i=1:data.nbtask
    fw=fw+data.C{i}*1/2*sum(max(0,1-data.Y{i}.*(data.X{i}*W(data.tasks==i))).^2);
end

costnew=fw+Omega1q_group(W(data.groups~=data.gbias),data.norm,data.groups(data.groups~=data.gbias));

if data.reg_sim
    costnew=costnew+data.reg_sim*cost_sim(W,data);
end

end


function res=cost_sim(W,data)

    nbtasks=data.nbtask;
    nbfeat=sum(data.tasks==1);
    
    W2=reshape(W,nbfeat,nbtasks);
    
    W2=W2(1:end-1,:);
    
    wm=mean(W2,2);
    
    dist=W2-repmat(wm,1,nbtasks);
    
    res=sum(sum(dist.*dist));

end

function res=grad_sim(W,data)

    nbtasks=data.nbtask;
    nbfeat=sum(data.tasks==1);
    
    W2=reshape(W,nbfeat,nbtasks);
    
    W2(end,:)=0;
    
    wm=mean(W2,2);
    
    dist=W2-repmat(wm,1,nbtasks);
    
    res=2*reshape(dist,nbtasks*nbfeat,1);

end


function Df=gradf(W,vars,data)

Df=zeros(size(W));

for i=1:data.nbtask
    Xt=data.X{i}.*repmat(data.Y{i},1,size(data.X{i},2));
    Df(data.tasks==i)=-data.C{i}*Xt'*max(0,1-data.Y{i}.*(data.X{i}*W(data.tasks==i)));
end

if data.reg_sim
    Df=Df+data.reg_sim*grad_sim(W,data);
end
% Df=-data.C*sum(data.X.*repmat(data.Y.*max(0,1-data.Y.*(data.X*W)),1,size(data.X,2)))';

end



function [W,obj,vars]=minq_func(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);

for i=1:length(vals)-1
    if norm(Vt(data.groups==vals(i)))>lambda
        W(data.groups==vals(i))=(1-lambda/norm(Vt(data.groups==vals(i)))).*Vt(data.groups==vals(i));
    end
end

W(data.groups==vals(end))=Vt(data.groups==vals(end));


obj=0;

end

function [W,obj,vars]=minq_func_1(Vt,vars,data)

lambda=1/vars.L;

vals=unique(data.groups);

%W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

% l1 proximal
W=(1-lambda./abs(Vt)).*Vt.*(abs(Vt)>lambda);

W(data.groups==vals(end))=Vt(data.groups==vals(end));

obj=0;

end

function [W,obj,vars]=minq_func_inf(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);

% l-inf proximal see Chen 2009
for i=1:length(vals)-1
    if norm(Vt(data.groups==vals(i)),data.popt)>lambda
        v=Vt(data.groups==vals(i));
        u=abs(v);
        u=sort(u,'descend');
        ur=cumsum(u);
        id=(1:length(v))';
        jvec=lambda-ur+id.*u;
        idtemp=find(jvec>0);
        val=(ur(idtemp(end))-lambda)/idtemp(end);
        W(data.groups==vals(i))=sign(v).*min(abs(v),val);
    end
end

W(data.groups==vals(end))=Vt(data.groups==vals(end));

obj=0;

end

function [W,obj,vars]=minq_func_q(Vt,vars,data)

lambda=1/vars.L;

W=zeros(size(Vt));
Vt=Vt-1/vars.L*gradf(Vt,vars,data);

vals=unique(data.groups);
opt.threshold=data.thr_prox;
% l-inf proximal see Chen 2009
for i=1:length(vals)-1
    if norm(Vt(data.groups==vals(i)),data.popt)>lambda

        W(data.groups==vals(i))=LSquareLq(Vt(data.groups==vals(i)),data.norm,lambda,opt);
    end
end

W(data.groups==vals(end))=Vt(data.groups==vals(end));

obj=0;

end



function constraints=get_constraints(W,data)
% illisible mais ça marche...

vals=unique(data.groups);

for i=1:length(vals)-1

    ri=zeros(sum(data.groups==vals(i)),1);
    
    cg=data.groups==vals(i);
    
    ct=data.tasks(cg);
    
    for j=1:data.nbtasks
     groups=data.groups(data.tasks==j);
     
    ri(ct==j)=data.C{j}*(data.X{j}(:,groups==vals(i)).*repmat(data.Y{j},1,sum(groups==vals(i))))'*max(0,1-data.Y{j}.*(data.X{j}*W(data.tasks==j)));
    end
    constraints(i)=norm(ri,data.popt);
end


end





