function [xsup,w,d,pos,timeps,alpha,obj]=svmclasslib(x,y,C,lambda,kernelt,kerneloption,verbose,span, alphainit,options)
% USAGE [xsup,w,b,pos,timeps,alpha,obj]=svmclass(x,y,c,lambda,kernel,kerneloption,verbose,span, alphainit)
%
% Support vector machine for CLASSIFICATION
% This routine classify the training set with a support vector machine
% using quadratic programming algorithm (active constraints method)
%
% INPUT
%
% Training set
%      x  		: input data 
%      y  		: output data
% parameters
%		c		: Bound on the lagrangian multipliers     
%		lambda		: Conditioning parameter for QP method
%		kernel		: kernel  type. classical kernel are
%
%		Name			parameters
%		'poly'		polynomial degree
%		'gaussian'	gaussian standard deviation
%
%		for more details see svmkernel
% 
%		kerneloption : parameters of kernel
%
%		for more details see svmkernel
%
% 		verbose : display outputs (default value is 0: no display)
%
%     Span    : span matrix for semiparametric learning 
%               This vector is sized Nbapp*Nbbasisfunction where
%               phi(i,j)= f_j(x(i));
%
%
%
%
% OUTPUT
%
% xsup	coordinates of the Support Vector
% w      weight
% b		bias
% pos    position of Support Vector
% timeps time for processing the scalar product
% alpha  Lagragian multiplier
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval

%	21/09/97 S. Canu
%	04/06/00 A. Rakotomamonjy   -inclusion of other kernel functions
%	04/05/01 S. Canu            -inclusion of multi-constraint optimization for frame-SVM
%
%       scanu@insa-rouen.fr, alain.rakoto@insa-rouen.fr

if nargin <10
        options=struct;
end

%if verbose ~= 0 disp('building the distance matrix'); end;
if ~isfield(options,'adaptiveC'); options.adaptiveC=0; end
if ~isfield(options,'w1'); options.w1=1/sum(y==1); end
if ~isfield(options,'w_1'); options.w_1=1/sum(y==-1); end


% libsvm retourne le pb sinon...
if y(1)>0
    sigclass=1;
    
else
    sigclass=-1;
    
    temp=options.w1;
    options.w1=options.w_1;
    options.w_1=temp;
    
end




    

switch kernelt
    case  'gaussian'
        kerneltype=2;
    case  'linear'
        kerneltype=0;
        degree=1;
    case  'poly'
        kerneltype=1;
    case 'numerical'
        kerneltype=4;
        x=[(1:length(y))' kerneloption.matrix];
    otherwise
        kerneltype=4;
        K=svmkernel(x,options.kernel,options.kerneloption);
        x=[(1:length(y))' K.matrix];
 end;
coefficient=0;
gamma=0;
degree=0;
if kerneltype==2
    degree=0;
    gamma= 1/2/kerneloption.^2;
elseif kerneltype==1 & kerneloption==1;
    coefficient=0;
    degree=1;
    gamma=1;
elseif kerneltype==1 & kerneloption>1;
    coefficient=1;
    degree=kerneloption;
    gamma=1;
    
else
    %test
    
end;
Cache=40;
epsilon=1e-7;
svmtype=0;

%parameters=[kerneltype degree gamma coefficient C Cache epsilon svmtype];
if options.adaptiveC
    
    
param=sprintf('-t %d -d %d -g %d -r %d -c %d -m %d -e %d -s %d -w1 %d -w-1 %d',kerneltype,degree,gamma,coefficient,C,Cache,epsilon,svmtype,options.w1,options.w_1);   
else
param=sprintf('-t %d -d %d -g %d -r %d -c %d -m %d -e %d -s %d',kerneltype,degree,gamma,coefficient,C,Cache,epsilon,svmtype);
end
model = svmtrain(y, x, param);

xsup=full(model.SVs);
w=sigclass*model.sv_coef;
d=-sigclass*model.rho;

%pos=[];
pos=model.pos+1;
timeps=0;
alpha=abs(w);

% it=1;
% for i=1:length(w)
%     if sum(abs(xsup(it,:)==x(i,:)))>1e-5
%         pos(it)=i;
%         it=it+1;
%     end
%     
% end

obj=-model.obj;

%K=svmkernel(xsup,kernelt,kerneloption);
%obj=-0.5*w'*K*w +sum(alpha);

% alphaall=zeros(size(e));
% alphaall(pos)=alpha;
% %obj=-0.5*alphaall'*H*alphaall +e'*alphaall;
% obj=0;
% if ~isempty(x)
%     xsup = x(pos,:);
% else
%     xsup=[];
% end;
% 
% ysup = y(pos);
% 
% 
% 
% w = (alpha.*ysup);
% d = svModel.model.mu{1};

end




