function [lda,obj]=xdawnclass(X,Y,options)
% USAGE [lda]=xdawnclass(X,Y,options)
%
%   performs a xDAWN spatial filtering prior to LDA classification on
%   windowed ERP examples 
%
%   Inputs
%   X : X \in R^{n \times d} each line contains the training exaample (vectorized)
%   Y : Y \in {-1,1}^n each line contains the label of the trainin example
%   options :
%       .lambda : regularisation term
%       .nbkeep : size of subspace kept for classification
%       .l_sig : length of the window of signal in samples (used for reshape)
%       .target_stimulus : label of the target stimulus
%       .U : pre-computed spatial filters
%
%       multiuclass is handled badly as one xDawn/classifier is done per
%       class using a one against all stagegy.

%	09/07/2011 R. Flamary




if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'nbkeep'); options.nbkeep=5; end
if ~isfield(options,'l_sig'); error('options.l_sig , length of the signal has to be set') ;end
if ~isfield(options,'target_stimulus'); options.target_stimulus=1; end
if ~isfield(options,'C'); options.C=1/options.lambda; end
if ~isfield(options,'U'); options.U=[]; end
if ~isfield(options,'multiclass'); options.multiclass=0; end

if length(unique(Y))>2
    options.multiclass=1;
end

if options.multiclass==0

    if nargin>3
        options.alphainit=alphainit;
    end

    options.C=options.C/length(Y);
        l_sig=options.l_sig;
        
    if isempty(options.U)
        % not god at all : do not take into account 

    
    % get the index of the target stimuli
    vals=unique(Y);
    target_stimulus=find(vals==options.target_stimulus);
    
    % transform everything back to signal*sensor 
    [Xout,index]=examples2sigs(X,Y,l_sig);
    
    [synchronisedResponses, enhancedResponse, DTarget, D] = xDAWN(Xout, index, target_stimulus);
    
    U=enhancedResponse.spatialFilter;
    
    else

       U=options.U; 
    end
    
    U=U(:,1:options.nbkeep);
    
    P=U2P(U,l_sig);
    
    X2=X*P;
    
    [w,w0]=ldaclass(X2,Y,1/options.C);
    obj=0;
    
    % save P and small w for fun
    lda.P=P;
    lda.wP=w;
    lda.xsup=[];
    % save the correct low dimensionnal linear classifier
    lda.w=P*w;
    lda.w0=w0;
    lda.pos=[];
    lda.kerneloption=1;
    lda.kernel='linear';

    
    
    
else

    vals=sort(unique(Y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;

            for i=1:length(vals)

                if ~isempty(options.alphainit)
                    options2.alphainit=options.alphainit{i};
                end


                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(Y==vals(i))-(Y~=vals(i));
                [models{i},obj(i)]=xdawnclass(X,Yt,options2);

            end

            lda.multiclass=options.multiclass;
            lda.nbclass=length(vals);
            lda.models=models;
            lda.vals=vals;




    end

end

end

function P=U2P(U,l_sig)
    
    [d,p]=size(U);
    
   % P=sparse(d*l_sig,p*l_sig);
    P=zeros(d*l_sig,p*l_sig);
    idx=1:l_sig:d*l_sig;
    
    for i=1:p
       
        for j=0:l_sig-1
            P(idx+j,1+j+(i-1)*l_sig)=U(:,i);
        end
        
    end

end







