function [prec_disc, prec_rejD, prec_rejAD, prec_rejAD_2, prec_CSR] = get_precision(vect_ytest , mat_ypred_val, classif_labels, ambiguity_treshold, distance_treshold, CSR_treshold)
% USAGE function [prec_disc, prec_rej,  prec_rejAD, prec_rejAD_2]= get_precision(vect_ytest, mat_ypred_val, classif_labels)
%
% vect_ytest est un vecteur donnant le label de chaque individu
% mat_ypred_val est la matrice donnant la confiance de l'appartenantce de chaque individu dans chaque classe
%
% #Facultatif
% classif_labels   est la liste des classes apprises.
%                    Par exemple: [1 2 5]. Il n'y a que 3 classes de sortie possibles, le modele ne sait pas reconnaitre les classes 3 et 4.
% ambiguity_treshold    seuil de rejet en ambiguite sur les valeurs de yval=p(Ci|x)       selon Chow
% distance_treshold     seuil de rejet en distance sur la valeur du max des yval=p(Ci|x). selon 
% CSR_treshold          seuil de rejet en ambiguite : "Class Selective Rejection rule"    selon Tien M. Ha
%
% CREATION
% 07 2009     B.LABBE
%
% MODIFICATIONS
% 09 2009     B.LABBE
%       ajout de la possibilite d'avoir des enssmble de classes apprises et testées différentes

test_labels=unique(vect_ytest);
labels= test_labels;
nb_indiv = length(vect_ytest);
%nb_class = length(labels);
if nargin <3
classif_labels=test_labels;
end
nb_class_app = length( classif_labels);
nb_class_test = length( test_labels);

%---------------------------------------------
[temp, ycalc]=max(mat_ypred_val,[],2);
ycalc=classif_labels(ycalc);
ycalc=ycalc(:);
%---------------------------------------------
prec_disc.CC=sum(ycalc==vect_ytest)/nb_indiv;
prec_disc.EC=1-prec_disc.CC;
prec_disc.labels=classif_labels;
prec_disc.labels_est=classif_labels;
prec_disc.labels_true=labels;
prec_disc.n_avg=1;
for i=1:nb_class_test
  prec_disc.cc(i)=sum(ycalc==labels(i) & vect_ytest==labels(i))/ sum(vect_ytest==labels(i));
end

prec_disc.matrice=zeros(nb_class_test,nb_class_app);
for i=1:nb_class_test
  for j=1:nb_class_app
    prec_disc.matrice(i,j)=sum(ycalc==classif_labels(j) & vect_ytest==labels(i));
  end
end
%---------------------------------------------


if nargout>1
    if nargin<5
        distance_treshold=1e-2;
    end
%---------------------------------------------
ypred_D=[ mat_ypred_val distance_treshold*ones(nb_indiv,1)];
[temp, ycalc]=max(ypred_D,[],2);
%ycalc(ycalc~=nb_class+1)=labels(ycalc(ycalc~=nb_class+1));
ycalc(ycalc~=nb_class_app+1)=classif_labels(ycalc(ycalc~=nb_class_app+1));

prec_rejD.CC=sum(ycalc==vect_ytest)/nb_indiv;
prec_rejD.EC=[];
prec_rejD.rejet_distance=sum(ycalc==nb_class_app+1)/nb_indiv;
prec_rejD.EC=round(1e5*(1-prec_rejD.CC-prec_rejD.rejet_distance))/1e5;
prec_rejD.labels=classif_labels;
prec_rejD.labels_est=classif_labels;
prec_rejD.labels_true=labels;
prec_rejD.n_avg=1;
for i=1:nb_class_test 
  prec_rejD.cc(i)=sum(ycalc==labels(i) & vect_ytest==labels(i))/ sum(vect_ytest==labels(i));
end

prec_rejD.matrice=zeros(nb_class_test,nb_class_app+1);
for i=1:nb_class_test
  for j=1:nb_class_app
    prec_rejD.matrice(i,j)=sum(ycalc==classif_labels(j) & vect_ytest==labels(i));
  end
  prec_rejD.matrice(i,nb_class_app+1)=sum(ycalc==nb_class_app+1 & vect_ytest==labels(i));
end
%---------------------------------------------
end

if nargout>2
%---------------------------------------------
ypred_AD=[mat_ypred_val distance_treshold*ones(nb_indiv,1)];
[temp, ycalc]=max(ypred_AD,[],2);
ycalc(sum(mat_ypred_val>distance_treshold   ,2)>1)=nb_class_app+2;
%ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2)=labels(ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2));
ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2)=classif_labels(ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2));

prec_rejAD.CC=sum(ycalc==vect_ytest)/nb_indiv;
prec_rejAD.EC=[];
prec_rejAD.rejet_distance=sum(ycalc==nb_class_app+1)/nb_indiv;
prec_rejAD.rejet_ambiguite=sum(ycalc==nb_class_app+2)/nb_indiv;
prec_rejAD.EC=round(1e5*(1-prec_rejAD.CC-prec_rejAD.rejet_ambiguite-prec_rejAD.rejet_distance) )/1e5;
prec_rejAD.labels=classif_labels;
prec_rejAD.labels_est=classif_labels;
prec_rejAD.labels_true=labels;
prec_rejAD.n_avg=1;
for i=1:nb_class_test
  prec_rejAD.cc(i)=sum(ycalc==labels(i) & vect_ytest==labels(i))/ sum(vect_ytest==labels(i));
end

prec_rejAD.matrice=zeros(nb_class_test,nb_class_app+2);
for i=1:nb_class_test
  for j=1:nb_class_app
    prec_rejAD.matrice(i,j)=sum(ycalc==classif_labels(j) & vect_ytest==labels(i));
  end
  prec_rejAD.matrice(i,nb_class_app+2)=sum(ycalc==nb_class_app+1 & vect_ytest==labels(i));
  prec_rejAD.matrice(i,nb_class_app+1)=sum(ycalc==nb_class_app+2 & vect_ytest==labels(i));
end
%---------------------------------------------
end



if nargout>3
%---------------------------------------------
ypred_AD=[mat_ypred_val distance_treshold*ones(nb_indiv,1)];
[temp, ycalc]=max(ypred_AD,[],2);
ycalc(sum(mat_ypred_val>distance_treshold   ,2)>1)=nb_class_app+2;
[mat_ypred_val_sorted, ind_mat_sorted]=sort(mat_ypred_val,2,'descend');
if nargin<4
    ambiguity_treshold=1e-2;
end
ind_ambiguite = (mat_ypred_val_sorted(:,2)>distance_treshold & mat_ypred_val_sorted(:,1)-mat_ypred_val_sorted(:,2)>=0);
ind_decision_prudente = (mat_ypred_val_sorted(:,2)>distance_treshold & mat_ypred_val_sorted(:,1)-mat_ypred_val_sorted(:,2)>=ambiguity_treshold);
ycalc(ind_decision_prudente)=nb_class_app+3;
vect_decision_prudente=zeros(nb_indiv,1);
%vect_decision_prudente(ind_decision_prudente)=labels(ind_mat_sorted(ind_decision_prudente,1));
vect_decision_prudente(ind_decision_prudente)=classif_labels(ind_mat_sorted(ind_decision_prudente,1));
%
ycalc(sum(mat_ypred_val>0,2)>1)=nb_class_app+2;
%ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2 & ycalc~=nb_class+3)=labels(ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2 & ycalc~=nb_class+3));
ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2 & ycalc~=nb_class_app+3)=classif_labels(ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2 & ycalc~=nb_class_app+3));

prec_rejAD_2.CC=sum(ycalc==vect_ytest)/nb_indiv;
prec_rejAD_2.EC=[];
prec_rejAD_2.rejet_distance=sum(ycalc==nb_class_app+1)/nb_indiv;
prec_rejAD_2.rejet_ambiguite=sum(ycalc==nb_class_app+2)/nb_indiv;
prec_rejAD_2.EC=round(1e5*(1-prec_rejAD.CC-prec_rejAD.rejet_ambiguite-prec_rejAD.rejet_distance) )/1e5;
prec_rejAD_2.CC_prudente=sum(vect_decision_prudente==vect_ytest)/sum(ind_ambiguite ~=0);
prec_rejAD_2.EC_prudente=sum(vect_decision_prudente~=0 & vect_decision_prudente~=vect_ytest)/sum(ind_ambiguite ~=0);
prec_rejAD_2.labels=classif_labels;
prec_rejAD_2.labels_est=classif_labels;
prec_rejAD_2.labels_true=labels;
prec_rejAD_2.n_avg=1;
for i=1:nb_class_test
  prec_rejAD_2.cc(i)=sum(ycalc==labels(i) & vect_ytest==labels(i))/ sum(vect_ytest==labels(i));
end

prec_rejAD_2.matrice=zeros(nb_class_test,nb_class_app+2);
prec_rejAD_2.matrice_prudente=zeros(nb_class_app,nb_class_app);
for i=1:nb_class_test
  for j=1:nb_class_app
    prec_rejAD_2.matrice(i,j)=sum(ycalc==classif_labels(j) & vect_ytest==labels(i));
    prec_rejAD_2.matrice_prudente(i,j)=sum(vect_decision_prudente==classif_labels(j) & vect_ytest==labels(i));
  end
  prec_rejAD_2.matrice(i,nb_class_app+2)=sum(ycalc==nb_class_app+1 & vect_ytest==labels(i));
  prec_rejAD_2.matrice(i,nb_class_app+1)=sum(ycalc==nb_class_app+2 & vect_ytest==labels(i));
end

%---------------------------------------------
end


if nargout>4
    if CSR_treshold~=-1
        %---------------------------------------------
        % ypred_CSR=[ mat_ypred_val zeros(nb_indiv,1) ];
        ypred_CSR=[ mat_ypred_val ];
        [ m_dec , p_val, n_avg ] = getOptimalDecision_CSR( ypred_CSR , 1/2*CSR_treshold );
        m_dec(m_dec(:,1)==0 , 1) =nb_class_app+1;
        ycalc = m_dec (: , 1);

        ind_non_rejet = find(sum(m_dec >0    ,2)<=1);
        
        ind_rejet_ambig = find(sum(m_dec >0    ,2)>1);
        ycalc(ind_rejet_ambig)=nb_class_app+2;
        %ycalc(sum(mat_ypred_val >distance_treshold    ,2)>1)=nb_class_app+2;

        %ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2)=labels(ycalc(ycalc~=nb_class+1 & ycalc~=nb_class+2));
        ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2)=classif_labels(ycalc(ycalc~=nb_class_app+1 & ycalc~=nb_class_app+2));

        prec_CSR.CC=sum( m_dec(ind_non_rejet,1) == vect_ytest(ind_non_rejet)   )/length(ind_non_rejet);
        prec_CSR.EC=[];
        prec_CSR.rejet_distance=sum(ycalc==nb_class_app+1)/nb_indiv;
        prec_CSR.rejet_ambiguite=sum(ycalc==nb_class_app+2)/nb_indiv;
        prec_CSR.EC=round(1e5*(1-prec_CSR.CC) )/1e5;
        prec_CSR.labels=classif_labels;
        prec_CSR.labels_est=classif_labels;
        prec_CSR.labels_true=labels;
        %
        prec_CSR.CC_ambiguite =sum(sum( m_dec(ind_rejet_ambig,:) == repmat(vect_ytest(ind_rejet_ambig), 1,nb_class_app ),2)  )/length(ind_rejet_ambig);
        prec_CSR.EC_ambiguite = 1-prec_CSR.CC_ambiguite;
        %
        for i=1:nb_class_test
          prec_CSR.cc(i)=sum(ycalc==labels(i) & vect_ytest==labels(i))/ sum(vect_ytest==labels(i));
        end

        prec_CSR.matrice=zeros(nb_class_test,nb_class_app+2);
        for i=1:nb_class_test
          for j=1:nb_class_app
            prec_CSR.matrice(i,j)=sum(ycalc==classif_labels(j) & vect_ytest==labels(i));
          end
          prec_CSR.matrice(i,nb_class_app+2)=sum(ycalc==nb_class_app+1 & vect_ytest==labels(i));
          prec_CSR.matrice(i,nb_class_app+1)=sum(ycalc==nb_class_app+2 & vect_ytest==labels(i));
        end
        %---
        prec_CSR.m_dec=m_dec;
        prec_CSR.p_val=p_val;
        prec_CSR.n_avg=n_avg;
        %---
    else
        prec_CSR=[];
    end
%---------------------------------------------
end
